/***************************************************************************
                          dcconnectionmanager.cpp  -  description
                             -------------------
    begin                : Mon Oct 1 2001
    copyright            : (C) 2001-2005 by Mathias Kster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qapplication.h>
#include <qobject.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcursor.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qinputdialog.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qtoolbar.h>
//Added by qt3to4:
#include <QEvent>
#include <QMdiArea>
#include <QMdiSubWindow>

#include "dcclient.h"
#include "dciconloader.h"

#include "dcconnectionmanager.h"

#include <dclib/dcos.h>

DCConnectionManager * g_pConnectionManager = 0;

/** */
DCConnectionManager::DCConnectionManager( QWidget * parent ) : QWidget( parent )
{
	setupUi(this);
	
	// set default icon
	setWindowIcon( g_pIconLoader->GetPixmap(eiSERVER) );

	m_pMdiArea = qobject_cast<QMdiArea*>(parent);
	
	/* if (m_pMdiArea != 0)
	{
		m_pMdiArea->addSubWindow(this);
	} */

	m_pMessageList = new CList<CObject>();

	InitDocument();

	g_pConnectionManager = this;
}

/** */
DCConnectionManager::~DCConnectionManager()
{
	DCClient * Client = 0;

	g_pConnectionManager = NULL;

	m_Mutex.Lock();

	while( (Client=(DCClient*)m_pClientList->Next(0)) != 0 )
	{
		Client->removeEventFilter(this);
		m_pClientList->Remove(Client);
		Client->Disconnect();
	}

	if ( m_pMessageList )
	{
		delete m_pMessageList;
		m_pMessageList = 0;
	}
	
	m_ClientsInTabOrder.clear();

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::InitDocument()
{
	StringMap * map;

	m_pTabBar = 0;
	m_nSelectedTab = -1;
	
	// restore settings
	if ( g_pConfig->GetMap("CONNECTIONVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGHT"].toInt() > 0) )
		{
			setGeometry( (*map)["X"].toInt(), (*map)["Y"].toInt(), (*map)["WIDTH"].toInt(), (*map)["HEIGHT"].toInt() );
		}
	}

	connect( &m_Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	connect( m_pMdiArea, SIGNAL(subWindowActivated(QMdiSubWindow *)), this, SLOT(slotMdiSubWindowActivated( QMdiSubWindow *)) );
	
	m_Timer.setSingleShot( TRUE );
	m_Timer.start( 500 );
}

/** */
void DCConnectionManager::InitTabBar( QToolBar * toolbar )
{
	m_pTabBar = new QTabBar(toolbar);
	//toolbar->setStretchableWidget(m_pTabBar);
	toolbar->addWidget(m_pTabBar);
	connect(m_pTabBar, SIGNAL(currentChanged(int)), this, SLOT(slotTabSelected(int)));
	// hide empty tabbar
	m_pTabBar->hide();
}

/** */
void DCConnectionManager::slotTabSelected( int id )
{
	if ( (m_nSelectedTab != id) && (id >= 0) && (id < m_ClientsInTabOrder.size()) )
	{
		DCClient * client = m_ClientsInTabOrder.at(id);
		m_pTabBar->setTabIcon(id,QIcon());
		client->showNormal();
		client->raise();
		m_pMdiArea->setActiveSubWindow(client->GetMdiSubWindow());
	}
	
	m_nSelectedTab = id;
}

/** */
void DCConnectionManager::DeInitDocument()
{
	StringMap * map;

	// save hub view settings
	g_pConfig->GetMap("CONNECTIONVIEW",map);

	(*map)["X"]          = QString().setNum(x());
	(*map)["Y"]          = QString().setNum(y());
	(*map)["WIDTH"]      = QString().setNum(width());
	(*map)["HEIGHT"]     = QString().setNum(height());
	(*map)["VISIBLE"]    = QString().setNum(isVisible());
	(*map)["MAXIMIZED"]  = QString().setNum(isMaximized());
	(*map)["MINIMIZED"]  = QString().setNum(isMinimized());
	(*map)["ENABLED"]    = QString().setNum(isEnabled());

	if ( !isEnabled() )
	{
		setEnabled(TRUE);
	}
}

/** event filter */
bool DCConnectionManager::eventFilter(QObject* object, QEvent* event)
{
	if ( ((DCConnectionManager*)object != this) && ((event->type() == QEvent::Close) || (event->type() == QEvent::CaptionChange)) )
	{
		DCClient * dcclient = qobject_cast<DCClient*>(object);
		
		if ( event->type() == QEvent::Close )
		{
			// remove tab
			int i = m_ClientsInTabOrder.indexOf(dcclient);
			if ( i != -1 )
			{
				m_pTabBar->removeTab(i);
				m_ClientsInTabOrder.removeAt(i);
			}

			dcclient->removeEventFilter(this);

			if ( m_pTabBar->count() == 0 )
				m_pTabBar->hide();
			
			dcclient->Disconnect(TRUE);
			dcclient->DeInitDocument();

			RemoveHub(dcclient);
			
			// update current tab
			m_nSelectedTab = m_pTabBar->currentIndex();
			
			event->accept();
			return TRUE;
		}
		else if ( event->type() == QEvent::CaptionChange )
		{
			// change caption
			int i = m_ClientsInTabOrder.indexOf(dcclient);
			if ( i != -1 )
			{
				CString s = dcclient->GetHubName();
				if ( s.Length() > 20 )
				{
					s = s.Left(20) + "...";
				}
				
				m_pTabBar->setTabText( i, QString::fromAscii(s.Data()) );
			}
		}
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** widget from mdi area activated */
void DCConnectionManager::slotMdiSubWindowActivated( QMdiSubWindow * w )
{
	DCClient * client = 0;
	int id = m_pTabBar->currentIndex();
	
	if ( (id >= 0) && (id < m_ClientsInTabOrder.size()) )
	{
		client = m_ClientsInTabOrder.at(id);
	}

	if ( client && w )
	{
		// only if tab change
		if ( client->GetMdiSubWindow() != w )
		{
			// set current tab
			for ( int i = 0; i < m_ClientsInTabOrder.size(); i++ )
			{
				if ( m_ClientsInTabOrder.at(i)->GetMdiSubWindow() == w )
				{
					m_nSelectedTab = i;
					m_pTabBar->setCurrentIndex( i );
					m_pTabBar->setTabIcon( i, QIcon() );
				}
			}
		}
	}
}

/** */
void DCConnectionManager::HubEvent( DCClient * client )
{
	if ( !client )
		return;
	
	int i = m_ClientsInTabOrder.indexOf(client);
	if ( (i != -1) && (i != m_pTabBar->currentIndex()) )
	{
		m_pTabBar->setTabIcon( i, QIcon(g_pIconLoader->GetPixmap(eiMESSAGE)) );
	}
}

/** callback function */
int DCConnectionManager::DC_CallBack( CObject * Object )
{
	int err;

	m_Mutex.Lock();

	if ( Object != NULL )
	{
		if ( m_pMessageList != NULL )
		{
			m_pMessageList->Add(Object);
			err = 0;
		}
		else
		{
			err = -1;
		}
	}
	else
	{
		err = -1;
	}

	m_Mutex.UnLock();

	return err;
}

/** */
void DCConnectionManager::timerDone()
{
	CObject * Object;
	CDCMessage *DCMsg;
	int i;

	for(i=0;i<50;i++)
	{
		if ( m_Mutex.TryLock() == FALSE )
		{
			break;
		}

		if ( m_pMessageList != 0 )
		{
			Object = m_pMessageList->Next(0);

			if ( Object != 0 )
			{
				m_pMessageList->Remove(Object);
			}
		}
		else
		{
			Object = 0;
		}

		m_Mutex.UnLock();

		if ( Object == 0 )
		{
			break;
		}

		DCMsg = (CDCMessage*) Object;

		switch ( DCMsg->m_eType )
		{
			case DC_MESSAGE_CONNECT_CLIENT:
			{
				DCMessageConnectClient *msg = (DCMessageConnectClient*)Object;

				Connect( msg->m_sHubName, msg->m_sHubHost );

				break;
			}

			default:
			{
				break;
			}
		}

		if ( Object )
		{
			delete Object;
		}
	}

	m_Timer.setSingleShot( TRUE );
	m_Timer.start( 500 );
}

/** */
void DCConnectionManager::AutoConnect()
{
	DCConfigHubItem * hubitem = 0;
	CList<DCConfigHubItem> list;
	DCConfigHubProfile ConfigHubProfile;

	g_pConfig->GetBookmarkHubList(&list);

	while( (hubitem=list.Next(hubitem)) != 0 )
	{
		// check for autoconnect
		if ( g_pConfig->GetHubProfile( hubitem->m_sProfile, &ConfigHubProfile ) == TRUE )
		{
			if ( ConfigHubProfile.m_bAutoConnect == TRUE )
			{
				Connect( hubitem->m_sName, hubitem->m_sHost );
			}
		}
	}
}

/** */
void DCConnectionManager::Connect( CString hubname, CString server, bool sslconnect )
{
	int i;
	CString s;
	DCClient * client;

	if ( server == "" )
	{
		return;
	}

	if ( hubname == "" )
	{
		hubname = server;
	}

	m_Mutex.Lock();

	client = (DCClient*)GetHub(hubname,server);

	if ( client )
	{
		client->setFocus();
	}

	m_Mutex.UnLock();

	if ( client )
	{
		return;
	}

	if ( (i = server.Find(':')) >= 0 )
	{
		s = server.Left(i);
	}
	else
	{
		s = server;
	}

	client = new DCClient(m_pMdiArea,g_pConfig->GetRemoteEncoding(hubname,server));
	client->setObjectName(QString::fromAscii(s.Data()));
	client->setAttribute(Qt::WA_DeleteOnClose);
	
	client->installEventFilter(this);
	
	s = client->GetHubName();
	if ( s.Length() > 20 )
		s = s.Left(20) + "...";

	if ( m_pTabBar->count() == 0 )
		m_pTabBar->show();
	
	m_pTabBar->addTab( QIcon(), s.Data() );
	m_ClientsInTabOrder.append(client);

	client->setWindowTitle(QString::fromAscii(hubname.Data()));

	if ( !g_pConfig->GetOpenClientWindowsMinimized() )
	{
		client->showNormal();
		client->InitSizes();
		client->centreOnMdiArea();
		// update tabbar
		slotMdiSubWindowActivated(client->GetMdiSubWindow());
	}
	else
	{
		client->showMinimized();
		client->InitSizes(); // but it probably won't work properly
	}

	CConnectionManager::Connect(hubname,server,client,sslconnect);
}

/** */
void DCConnectionManager::OpenPrivateChat( QString hubname, QString hubhost, QString nick )
{
        DCClient * Client = 0;

	m_Mutex.Lock();

	Client = (DCClient*)GetHub(hubname.toAscii().constData(),hubhost.toAscii().constData());

	if ( Client )
	{
		Client->DC_PrivateChat(nick,"","",TRUE);
	}

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::CloseAllChats()
{
	CClient * Client = 0;

	m_Mutex.Lock();

	while( (Client=m_pClientList->Next(Client)) != 0 )
	{
		if ( (DCClient*)Client != 0 )
		{
			((DCClient*)Client)->CloseAllChats();
		}
	}

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::DisconnectAllClients()
{
	CClient * Client = 0;

	m_Mutex.Lock();

	while( (Client=m_pClientList->Next(Client)) != 0 )
	{
		m_Mutex.UnLock();
		if ( (DCClient*)Client != 0 )
		{
			((DCClient*)Client)->Disconnect(TRUE);
		}
		m_Mutex.Lock();
	}

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::CloseDisconnectedHubs()
{
	CClient * Client = 0, * oClient = 0;

	m_Mutex.Lock();

	while( (Client=m_pClientList->Next(Client)) != 0 )
	{
		if ( ((DCClient*)Client)->GetConnectionState() == estNONE )
		{
			m_Mutex.UnLock();
			((DCClient*)Client)->GetMdiSubWindow()->close();
			m_Mutex.Lock();

			Client = oClient;
		}
		else
		{
			oClient = Client;
		}
	}

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::OPKick( QString hubname, QString hubhost, QString nick, QString message )
{
        DCClient * Client = 0;

	m_Mutex.Lock();

	Client = (DCClient*)GetHub(hubname.toAscii().constData(),hubhost.toAscii().constData());

	if ( Client )
	{
		Client->OPKick( nick, message );
	}

	m_Mutex.UnLock();
}

/** */
void DCConnectionManager::OPForceMove( QString hubname, QString hubhost, QString nick, QString message, QString host )
{
        DCClient * Client = 0;

	m_Mutex.Lock();

	Client = (DCClient*)GetHub(hubname.toAscii().constData(),hubhost.toAscii().constData());

	if ( Client )
	{
		Client->OPForceMove( nick, message, host );
	}

	m_Mutex.UnLock();
}
