/***************************************************************************
                          dchublistmanager.cpp  -  description
                             -------------------
    begin                : Mon Oct 1 2001
    copyright            : (C) 2001-2004 by Mathias Kster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qapplication.h>
#include <qobject.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qcursor.h>
#include <qtabwidget.h>
#include <qinputdialog.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qclipboard.h>
//Added by qt3to4:
#include <QResizeEvent>
#include <QEvent>
#include <QMdiArea>
#include <QMdiSubWindow>
#include <QHeaderView>
#include <vector>

#include "dcmenuhandler.h"
#include "dchubprofile.h"
#include "dchubfilter.h"
#include "dciconloader.h"
#include "dcconnectionmanager.h"
#include "dcconfig.h"

#include "dchublistmanager.h"
#include "dchublistitems.h"
#include "dcguiutils.h"

#include <dclib/dcos.h>
#include <dclib/core/cstringlist.h>

#include "dceditserver.h"
#include "cdialogmessage.h"
#include "publichubsmodel.h"

DCHubListManager * g_pHubListManager = 0;

/** */
DCHubListManager::DCHubListManager( QWidget * parent ) : QWidget( parent )
{
	setupUi(this);

	lastPublicHubsWidth = -1;
	lastBookmarkHubsWidth = -1;

	/* we have custom column resizing code */
	TreeView_PUBLIC->header()->setStretchLastSection(FALSE);
	TreeWidget_BOOKMARKS->header()->setStretchLastSection(FALSE);

	// set default icon
	setWindowIcon( g_pIconLoader->GetPixmap(eiSERVER) );

	if ( (parent != 0) && (qobject_cast<QMdiArea*>(parent) != 0) )
	{
		m_pContainerWindow = new QMdiSubWindow();
		m_pContainerWindow->setWidget(this);
		(qobject_cast<QMdiArea*>(parent))->addSubWindow(m_pContainerWindow);
		m_pContainerWindow->setWindowIcon( g_pIconLoader->GetPixmap(eiSERVER) );
	}
	else
	{
		m_pContainerWindow = 0;
	}

	model = new PublicHubsModel();
	TreeView_PUBLIC->setModel( model );
	
	m_pMessageList = new CList<CObject>();

	InitDocument();

	g_pHubListManager = this;
}

/** */
DCHubListManager::~DCHubListManager()
{
	g_pHubListManager = NULL;

	m_HLMMutex.Lock();

	if ( m_pMessageList )
	{
		delete m_pMessageList;
		m_pMessageList = 0;
	}

	m_HLMMutex.UnLock();
	
	delete model;
	
	// we might have removed public hubs
	if ( g_pConfig->GetRemovePublicHubEnabled() )
	{
		g_pConfig->SaveDCPublicHub();
	}
	
	// bookmarks may have been sorted and the new order needs to be saved
	updateBookmarkOrder();
	
	/* delete all hub filter objects */
	for ( QMap<QString, DC_HubFilterObject*>::const_iterator it = m_HubFilterMap.constBegin(); it != m_HubFilterMap.constEnd(); ++it )
	{
		delete it.value();
	}
	m_HubFilterMap.clear();
}

/** current tab widget change slot */
void DCHubListManager::slotTabWidgetCurrentChange(QWidget*)
{
	SizeColumnsPreservingRatios();
}

/** overridden so that the column widths are initialized on first show() */
void DCHubListManager::show()
{
	QWidget::show();

	SizeColumnsPreservingRatios();
}

/** resize event handler */
void DCHubListManager::resizeEvent( QResizeEvent * )
{
	SizeColumnsPreservingRatios();
}

/** Initialize or adjust widths of the ListView columns */
void DCHubListManager::SizeColumnsPreservingRatios()
{
	int width;
	
	if ( TreeView_PUBLIC->isVisible() )
	{
		width = TreeView_PUBLIC->width();
		if ( width > 0 )
		{
			if ( lastPublicHubsWidth == -1 )
			{
				/* set initial widths using default ratios */
				/* also expand the list view since we have lots of columns now */
				TreeView_PUBLIC->setColumnWidth( 0, ((width*3)/10) );
				TreeView_PUBLIC->setColumnWidth( 1, ((width*3)/10) );
				TreeView_PUBLIC->setColumnWidth( 2, ((width*3)/10) );
				TreeView_PUBLIC->setColumnWidth( 3, ((width*1)/10) );
				TreeView_PUBLIC->setColumnWidth( 4, ((width*2)/10) );
				TreeView_PUBLIC->setColumnWidth( 5, ((width*1)/10) );
				TreeView_PUBLIC->setColumnWidth( 6, ((width*1)/10) );
				TreeView_PUBLIC->setColumnWidth( 7, ((width*3)/10) );
				
				lastPublicHubsWidth = TreeView_PUBLIC->width();
			}
			else if ( lastPublicHubsWidth != width )
			{
				DCGuiUtils::AdjustColumnWidths( TreeView_PUBLIC, lastPublicHubsWidth );			
				lastPublicHubsWidth = TreeView_PUBLIC->width();
			}
		}
	}
	else if ( TreeWidget_BOOKMARKS->isVisible() )
	{
		width = TreeWidget_BOOKMARKS->width();
		if ( width > 0 )
		{
			if ( lastBookmarkHubsWidth == -1 )
			{
				TreeWidget_BOOKMARKS->setColumnWidth( 0, width/3 );
				TreeWidget_BOOKMARKS->setColumnWidth( 1, width/3 );
				TreeWidget_BOOKMARKS->setColumnWidth( 2, width/3 );
				
				lastBookmarkHubsWidth = TreeWidget_BOOKMARKS->width();
			}
			else if ( lastBookmarkHubsWidth != width )
			{
				DCGuiUtils::AdjustColumnWidths( TreeWidget_BOOKMARKS, lastBookmarkHubsWidth );
				lastBookmarkHubsWidth = TreeWidget_BOOKMARKS->width();
			}
		}
	}
}

/** */
void DCHubListManager::SetToolBar( QFrame * frame1, QFrame * frame2, bool vis )
{
	if ( vis )
	{
		frame1->hide();
		frame1->setEnabled(FALSE);
		frame2->show();
		frame2->setEnabled(TRUE);
	}
	else
	{
		frame2->hide();
		frame2->setEnabled(FALSE);
		frame1->show();
		frame1->setEnabled(TRUE);
	}
}

/** */
void DCHubListManager::InitDocument()
{
	StringMap * map;
	bool btb1=TRUE,btb2=TRUE;
	bool bfilter=FALSE;

	DC_HubFilterObject * HubFilterObject;

	// hide progressbar
	ProgressBar_LOADHUBLIST->setMinimum(0);
	ProgressBar_LOADHUBLIST->hide();

	// set bookmark icon
	TabWidget_SERVERLIST->setTabIcon( 1, QIcon( g_pIconLoader->GetPixmap(eiBOOKMARK_FOLDER) ) );

	// init the toolbars
	Frame_TOOLBAR1_HIDE->installEventFilter(this);
	Frame_TOOLBAR1_SHOW->installEventFilter(this);
	Frame_TOOLBAR2_HIDE->installEventFilter(this);
	Frame_TOOLBAR2_SHOW->installEventFilter(this);
	
	// restore settings
	if ( g_pConfig->GetMap("HUBVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGHT"].toInt() > 0) )
		{
			if ( m_pContainerWindow != 0 )
			{
				m_pContainerWindow->setGeometry( (*map)["X"].toInt(), (*map)["Y"].toInt(), (*map)["WIDTH"].toInt(), (*map)["HEIGHT"].toInt() );
			}
		}

		if ( ((*map)["TOOLBAR1"] != "") && ((*map)["TOOLBAR1"].toInt() == 0) )
		{
			btb1 = FALSE;
		}

		if ( ((*map)["TOOLBAR2"] != "") && ((*map)["TOOLBAR2"].toInt() == 0) )
		{
			btb2 = FALSE;
		}

		// restore active page
		TabWidget_SERVERLIST->setCurrentIndex((*map)["ACTIVEPAGE"].toInt());
		// restore linedit
		LineEdit_SERVER->setText((*map)["LE_SERVER"]);
		
		/* restore sorting parameters */
		TreeView_PUBLIC->sortByColumn( (*map)["PUBLICSORTCOLUMN"].toInt(), DCGuiUtils::SortOrderFromName((*map)["PUBLICSORTORDER"]) );
		
		if ( ((*map)["BOOKMARKSORTINGENABLED"] != "") && ((*map)["BOOKMARKSORTINGENABLED"].toInt() == 0) )
		{
			TreeWidget_BOOKMARKS->setSortingEnabled( FALSE );
		}
		else
		{
			TreeWidget_BOOKMARKS->setSortingEnabled( TRUE );
		}
		
		TreeWidget_BOOKMARKS->sortItems( (*map)["BOOKMARKSORTCOLUMN"].toInt(), DCGuiUtils::SortOrderFromName((*map)["BOOKMARKSORTORDER"]) );
	}

	SetToolBar( Frame_TOOLBAR1_HIDE, Frame_TOOLBAR1_SHOW, btb1 );
	SetToolBar( Frame_TOOLBAR2_HIDE, Frame_TOOLBAR2_SHOW, btb2 );

	connect( LineEdit_SERVER, SIGNAL(returnPressed()), this, SLOT(slotConnect()) );
	connect( ToolButton_RELOADPUBLICHUBLIST, SIGNAL(clicked()), this, SLOT(slotReloadPublicHubList()) );
	connect( ToolButton_UPDATEPUBLICHUBLIST, SIGNAL(clicked()), this, SLOT(slotUpdatePublicHubList()) );
	connect( ToolButton_CONNECT, SIGNAL(clicked()), this, SLOT(slotConnect()) );

	// public list
	connect( TreeView_PUBLIC,SIGNAL(activated( const QModelIndex & )), this, SLOT(slotItemActivatedPublic( const QModelIndex & )) );
	connect( TreeView_PUBLIC,SIGNAL(customContextMenuRequested( const QPoint & )), this, SLOT(slotContextMenuPublicList( const QPoint & )) );

	// bookmark list
	connect( TreeWidget_BOOKMARKS,SIGNAL(itemActivated( QTreeWidgetItem *, int )), this, SLOT(slotItemActivatedBookmarks( QTreeWidgetItem *, int)) );
	connect( TreeWidget_BOOKMARKS,SIGNAL(customContextMenuRequested( const QPoint & )), this, SLOT(slotContextMenuBookmarkList( const QPoint & )) );
	connect( ToolButton_BOOKMARK_UP, SIGNAL(clicked()), this, SLOT(slotMoveBookmarkUp()) );
	connect( ToolButton_BOOKMARK_DOWN, SIGNAL(clicked()), this, SLOT(slotMoveBookmarkDown()) );
	connect( ToolButton_BOOKMARK_ENABLE_SORT, SIGNAL(clicked()), this, SLOT(slotEnableBookmarkSorting()) );
	connect( ToolButton_BOOKMARK_SAVE, SIGNAL(clicked()), this, SLOT(slotSaveBookmarks()) );

	connect( TabWidget_SERVERLIST,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetCurrentChange(QWidget*)) );

	// filter
	connect( ToolButton_ADDFILTER, SIGNAL(clicked()), this, SLOT(slotAddFilter()) );
	connect( ToolButton_EDITFILTER, SIGNAL(clicked()), this, SLOT(slotEditFilter()) );
	connect( ToolButton_DELFILTER, SIGNAL(clicked()), this, SLOT(slotDelFilter()) );
	connect( ComboBox_FILTER, SIGNAL(activated( const QString & )), this, SLOT(slotFilterChange( const QString &)) );

	connect( &m_Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	// load filter
	g_pConfig->LoadDCHubFilter(&m_HubFilterMap);

	// create default
	if ( m_HubFilterMap.isEmpty() )
	{
		HubFilterObject = new DC_HubFilterObject();

		HubFilterObject->m_sFilterName = "Default";

		m_HubFilterMap.insert( HubFilterObject->m_sFilterName, HubFilterObject );

		g_pConfig->SaveDCHubFilter(&m_HubFilterMap);
	}

	for ( QMap<QString, DC_HubFilterObject*>::const_iterator it = m_HubFilterMap.constBegin(); it != m_HubFilterMap.constEnd(); ++it )
	{
		HubFilterObject = it.value();
		
		ComboBox_FILTER->addItem( HubFilterObject->m_sFilterName );

		if ( (*map)["FILTER"] == HubFilterObject->m_sFilterName )
		{
			bfilter = TRUE;
		}
	}

	// restore filter
	if ( bfilter )
	{
		ComboBox_FILTER->setEditText((*map)["FILTER"]);
	}
	else
	{
		ComboBox_FILTER->setEditText("Default");
	}

	ShowBookmarkList();
	ShowPublicHubList();

	m_Timer.setSingleShot( TRUE );
	m_Timer.start( 500 );
}

/** */
void DCHubListManager::DeInitDocument()
{
	StringMap * map;

	// save hub view settings
	g_pConfig->GetMap("HUBVIEW",map);

	if ( m_pContainerWindow != 0 )
	{
		(*map)["X"]          = QString().setNum(m_pContainerWindow->x());
		(*map)["Y"]          = QString().setNum(m_pContainerWindow->y());
		(*map)["WIDTH"]      = QString().setNum(m_pContainerWindow->width());
		(*map)["HEIGHT"]     = QString().setNum(m_pContainerWindow->height());
		(*map)["VISIBLE"]    = QString().setNum(m_pContainerWindow->isVisible());
		(*map)["MAXIMIZED"]  = QString().setNum(m_pContainerWindow->isMaximized());
		(*map)["MINIMIZED"]  = QString().setNum(m_pContainerWindow->isMinimized());
		(*map)["ENABLED"]    = QString().setNum(m_pContainerWindow->isEnabled());
	}
	(*map)["ACTIVEPAGE"] = QString().setNum(TabWidget_SERVERLIST->currentIndex());
	(*map)["LE_SERVER"]  = LineEdit_SERVER->text();
	(*map)["FILTER"]     = ComboBox_FILTER->currentText();

	if ( !isEnabled() )
	{
		setEnabled(TRUE);
	}

	(*map)["TOOLBAR1"]  = QString().setNum(Frame_TOOLBAR1_SHOW->isEnabled());
	(*map)["TOOLBAR2"]  = QString().setNum(Frame_TOOLBAR2_SHOW->isEnabled());
	
	int column = 0;
	Qt::SortOrder order = Qt::AscendingOrder;
	
	model->getSorting( column, order );
	
	(*map)["PUBLICSORTCOLUMN"] = QString().setNum(column);
	(*map)["PUBLICSORTORDER"]  = DCGuiUtils::SortOrderName( order );
	
	(*map)["BOOKMARKSORTCOLUMN"] = QString().setNum(TreeWidget_BOOKMARKS->sortColumn());
	(*map)["BOOKMARKSORTORDER"]  = DCGuiUtils::SortOrderName( TreeWidget_BOOKMARKS->header()->sortIndicatorOrder() );
	(*map)["BOOKMARKSORTINGENABLED"] = QString().setNum(TreeWidget_BOOKMARKS->isSortingEnabled());
}

/** event filter */
bool DCHubListManager::eventFilter(QObject* object, QEvent* event)
{
	if ( event->type() == QEvent::MouseButtonDblClick )
	{
		if ( object == Frame_TOOLBAR1_HIDE )
		{
			SetToolBar( Frame_TOOLBAR1_HIDE, Frame_TOOLBAR1_SHOW, TRUE );
		}
		else if ( object == Frame_TOOLBAR1_SHOW )
		{
			SetToolBar( Frame_TOOLBAR1_HIDE, Frame_TOOLBAR1_SHOW, FALSE );
		}
		else if ( object == Frame_TOOLBAR2_HIDE )
		{
			SetToolBar( Frame_TOOLBAR2_HIDE, Frame_TOOLBAR2_SHOW, TRUE );

			// show progressbar
			if ( m_pHubListUrl != 0 )
				ProgressBar_LOADHUBLIST->show();
		}
		else if ( object == Frame_TOOLBAR2_SHOW )
		{
			SetToolBar( Frame_TOOLBAR2_HIDE, Frame_TOOLBAR2_SHOW, FALSE );
		}
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** callback function */
int DCHubListManager::DC_CallBack( CObject * Object )
{
	int err;

	m_HLMMutex.Lock();

	if ( Object != NULL )
	{
		if ( m_pMessageList != NULL )
		{
			m_pMessageList->Add(Object);
			err = 0;
		}
		else
		{
			err = -1;
		}
	}
	else
	{
		err = -1;
	}

	m_HLMMutex.UnLock();

	return err;
}

/** */
void DCHubListManager::timerDone()
{
	CObject * Object;
	CDCMessage *DCMsg;
	int i;

	for(i=0;i<50;i++)
	{
		if ( m_HLMMutex.TryLock() == FALSE )
		{
			break;
		}

		if ( m_pMessageList != 0 )
		{
			Object = m_pMessageList->Next(0);

			if ( Object != 0 )
			{
				m_pMessageList->Remove(Object);
			}
		}
		else
		{
			Object = 0;
		}

		m_HLMMutex.UnLock();

		if ( Object == 0 )
		{
			break;
		}

		DCMsg = (CDCMessage*) Object;

		switch ( DCMsg->m_eType )
		{
			case DC_MESSAGE_GETHUBLIST:
			{
				DCMessageGetHubList * msg = (DCMessageGetHubList*) Object;

				if ( msg->m_bRun == FALSE )
				{
					// enable buttons
					ToolButton_RELOADPUBLICHUBLIST->setEnabled(TRUE);
					ToolButton_UPDATEPUBLICHUBLIST->setEnabled(TRUE);

					// hide progressbar
					ProgressBar_LOADHUBLIST->hide();

					ShowPublicHubList();
				}
				else
				{
					// disable buttons
					ToolButton_RELOADPUBLICHUBLIST->setEnabled(FALSE);
					ToolButton_UPDATEPUBLICHUBLIST->setEnabled(FALSE);

					// show progressbar
					ProgressBar_LOADHUBLIST->show();
				}

				break;
			}

			case DC_MESSAGE_TRANSFER:
			{
				CMessageTransfer *msg = (CMessageTransfer*)Object;

				ProgressBar_LOADHUBLIST->setMaximum(msg->m_nLength);
				ProgressBar_LOADHUBLIST->setValue(msg->m_nTransfered);

				break;
			}

			default:
			{
				break;
			}
		}

		if ( Object )
		{
			delete Object;
		}
	}

	m_Timer.setSingleShot( TRUE );
	m_Timer.start( 500 );
}

/** */
void DCHubListManager::ShowPublicHubList()
{
	DCConfigHubItem * hubitem;
	CList<DCConfigHubItem> PublicHubList;
	unsigned int minuser;
	DC_HubFilterObject * HubFilterObject = 0;
	QString s;
	bool bshow;
	QString AllFields,Pattern,Contains;
	int i,x=0;
	int sortColumn=0;
	Qt::SortOrder sortOrder = Qt::AscendingOrder;

	// disable filter combo box
	ComboBox_FILTER->setEnabled(FALSE);

	TabWidget_SERVERLIST->setTabText( 0, tr("Public")+" (0/0)" );

	TreeView_PUBLIC->setUpdatesEnabled(FALSE);
	TreeView_PUBLIC->setSortingEnabled(FALSE);
	TreeView_PUBLIC->clearSelection();
	model->getSorting( sortColumn, sortOrder );
	model->clear();

	g_pConfig->GetPublicHubList(&PublicHubList);

	hubitem     = 0;
	minuser     = 0;
	
	// get the hubfilter
	s = ComboBox_FILTER->currentText();

	if ( !(s.isEmpty()) )
	{
		HubFilterObject = m_HubFilterMap.value( s );

		if ( HubFilterObject &&
		     !HubFilterObject->m_bName &&
		     !HubFilterObject->m_bServer &&
		     !HubFilterObject->m_bDescription &&
		     (HubFilterObject->m_nUser==0) )
		{
			HubFilterObject = 0;
		}
	}

	while( (hubitem=PublicHubList.Next(hubitem)) != 0 )
	{
		bshow = TRUE;

		x++;

		if ( x == 100 )
		{
			qApp->processEvents();
			x = 0;
		}

		// use the filter
		if ( HubFilterObject )
		{
			if ( hubitem->m_sUserCount.asULL() < HubFilterObject->m_nUser )
			{
				bshow = FALSE;
			}
			else
			{
				AllFields = QString::null;

				if ( HubFilterObject->m_bName )
					AllFields += hubitem->m_sName.Data();
				if ( HubFilterObject->m_bServer )
					AllFields += hubitem->m_sHost.Data();
				if ( HubFilterObject->m_bDescription )
					AllFields += hubitem->m_sDescription.Data();

				Contains = HubFilterObject->m_sContains;

				if ( AllFields != Contains )
				{
					i = 0;
					while ( !(Pattern=Contains.section(" ",i,i,QString::SectionSkipEmpty)).isEmpty())
					{
						i++;
						if ( !(AllFields.contains(Pattern,Qt::CaseInsensitive)) )
						{
							bshow = FALSE;
							break;
						}
					}
				}
			}
		}

		if ( bshow )
		{
			model->addHub(
				QString::fromAscii(hubitem->m_sName.Data()),
				QString::fromAscii(hubitem->m_sHost.Data()),
				QString::fromAscii(hubitem->m_sDescription.Data()),
				hubitem->m_sUserCount.asULL(),
				QString::fromAscii(hubitem->m_sCountry.Data()),
				hubitem->m_nShared,
				hubitem->m_nMinShare,
				QString::fromAscii(hubitem->m_sExtra.Data())
			);
		}
	}

	TreeView_PUBLIC->setSortingEnabled(TRUE);
	TreeView_PUBLIC->sortByColumn( sortColumn, sortOrder );
	TreeView_PUBLIC->setUpdatesEnabled(TRUE);

	TabWidget_SERVERLIST->setTabText( 0, tr("Public")+" ("+QString().setNum(model->rowCount( QModelIndex() ))+"/"+QString().setNum(PublicHubList.Count())+")");

	// enable filter combo box
	ComboBox_FILTER->setEnabled(TRUE);
}

/** */
void DCHubListManager::ShowBookmarkList()
{
	int x = 0;
	DCConfigHubItem * hubitem = 0;
	CList<DCConfigHubItem> list;
	bool sortingenabled = TreeWidget_BOOKMARKS->isSortingEnabled();

	g_pConfig->GetBookmarkHubList(&list);

	TreeWidget_BOOKMARKS->setSortingEnabled(FALSE);
	TreeWidget_BOOKMARKS->setUpdatesEnabled(FALSE);
	TreeWidget_BOOKMARKS->clear();

	while( (hubitem=list.Next(hubitem)) != 0 )
	{
		DCBookmarkHubItem * item = new DCBookmarkHubItem( TreeWidget_BOOKMARKS );
		item->setText( 0, hubitem->m_sName.Data() );
		item->setText( 1, hubitem->m_sHost.Data() );
		item->setText( 2, hubitem->m_sDescription.Data() );
		item->position = x;

		x++;

		if ( x == 100 )
		{
			qApp->processEvents();
			x = 0;
		}
	}

	if ( sortingenabled )
	{
		TreeWidget_BOOKMARKS->setSortingEnabled(sortingenabled);
	}
	TreeWidget_BOOKMARKS->setUpdatesEnabled(TRUE);

	TabWidget_SERVERLIST->setTabText( 1, tr("Bookmarks")+" ("+QString().setNum(TreeWidget_BOOKMARKS->model()->rowCount())+")" );
}

/** */
void DCHubListManager::slotReloadPublicHubList()
{
	g_pConfig->ClearPublicHubList();

	GetPublicHubList();
}

/** update public hub list */
void DCHubListManager::slotUpdatePublicHubList()
{
	GetPublicHubList();
}

/** */
void DCHubListManager::slotItemActivatedPublic( const QModelIndex & index )
{
	PublicHubItem * item = model->getItem( index.row() );
	
	if ( item )
		g_pConnectionManager->Connect( item->name.toAscii().constData(), item->server.toAscii().constData() );
}

/** */
void DCHubListManager::slotItemActivatedBookmarks( QTreeWidgetItem * item, int /* column */ )
{
	if ( item )
		g_pConnectionManager->Connect( item->text(0).toAscii().constData(), item->text(1).toAscii().constData() );
}

/** */
void DCHubListManager::slotConnect()
{
	g_pConnectionManager->Connect( "", LineEdit_SERVER->text().toAscii().constData() );
}

/** */
void DCHubListManager::slotAddFilter()
{
	DC_HubFilterObject * HubFilterObject;
	DCHubFilter * hf;

	hf = new DCHubFilter(this);

	if ( hf->exec() == QDialog::Accepted )
	{
		HubFilterObject = new  DC_HubFilterObject();

		// get the new data
		hf->Get( HubFilterObject );

		if ( HubFilterObject->m_sFilterName.isEmpty() )
		{
			// todo: error message

			delete HubFilterObject;
		}
		else if ( m_HubFilterMap.contains( HubFilterObject->m_sFilterName ) )
		{
			QMessageBox::information(
				this,
				tr("Add hub filter"),
				tr("A filter with that name already exists")
			);

			delete HubFilterObject;
		}
		else
		{
			m_HubFilterMap.insert( HubFilterObject->m_sFilterName, HubFilterObject );

			ComboBox_FILTER->addItem( HubFilterObject->m_sFilterName );
		}

		g_pConfig->SaveDCHubFilter(&m_HubFilterMap);

		delete hf;
	}
}

/** */
void DCHubListManager::slotEditFilter()
{
	QString s;
	DC_HubFilterObject * HubFilterObject;
	DCHubFilter * hf;

	s = ComboBox_FILTER->currentText();

	if ( (!(s.isEmpty())) && (s != "Default") )
	{
		HubFilterObject = m_HubFilterMap.value( s );
		if ( HubFilterObject != 0 )
		{
			hf = new DCHubFilter(this);

			hf->Edit( HubFilterObject );

			if ( hf->exec() == QDialog::Accepted )
			{
				// get the new data
				hf->Get( HubFilterObject );

				if ( HubFilterObject->m_sFilterName.isEmpty() )
				{
					HubFilterObject->m_sFilterName = s;

					// todo: error message
				}
				else if ( s != HubFilterObject->m_sFilterName )
				{
					if ( m_HubFilterMap.contains( HubFilterObject->m_sFilterName ) )
					{
						QMessageBox::information(
							this,
							tr("Edit hub filter"),
							tr("A filter with that name already exists")
						);
						HubFilterObject->m_sFilterName = s;
					}
					else
					{
						/* not deleted because it's the same pointer */
						m_HubFilterMap.remove(s);
						m_HubFilterMap.insert( HubFilterObject->m_sFilterName, HubFilterObject );

						ComboBox_FILTER->setItemText( ComboBox_FILTER->currentIndex(), HubFilterObject->m_sFilterName );
					}
				}

				// update view
				slotFilterChange( HubFilterObject->m_sFilterName );

				g_pConfig->SaveDCHubFilter(&m_HubFilterMap);
			}

			delete hf;
		}
	}
}

/** */
void DCHubListManager::slotDelFilter()
{
	QString s = ComboBox_FILTER->currentText();

	if ( (!(s.isEmpty())) && (s != "Default") )
	{
		ComboBox_FILTER->removeItem(ComboBox_FILTER->currentIndex());

		QMap<QString, DC_HubFilterObject*>::const_iterator it = m_HubFilterMap.find( s );
		if ( it != m_HubFilterMap.constEnd() )
		{
			delete it.value();
			m_HubFilterMap.remove( s );
			g_pConfig->SaveDCHubFilter(&m_HubFilterMap);
		}
		else
		{
			QMessageBox::warning(
				this,
				tr("Delete hub filter"),
				tr("Filter not found")
			);
		}
	}
}

/** */
void DCHubListManager::slotFilterChange( const QString & s )
{
	if ( !(s.isEmpty()) )
	{
		ShowPublicHubList();
	}
}

/** */
void DCHubListManager::RemoveBookmark( QTreeWidgetItem * item )
{
	if ( g_pConfig->RemoveBookmarkHub( item->text(0).toAscii().constData(), item->text(1).toAscii().constData(), item->text(2).toAscii().constData() ) == TRUE )
	{
		g_pConfig->SaveDCBookHub();
		
		QTreeWidgetItemIterator it( TreeWidget_BOOKMARKS );
		DCBookmarkHubItem * removed = (DCBookmarkHubItem*) item;
		for ( ; (*it); ++it )
		{
			DCBookmarkHubItem * current = (DCBookmarkHubItem*) (*it);
			if ( current->position > removed->position )
			{
				current->position--;
			}
		}

		delete item;

		// update tabbar
		TabWidget_SERVERLIST->setTabText( 1, tr("Bookmarks")+" ("+QString().setNum(TreeWidget_BOOKMARKS->model()->rowCount())+")" );
	}
}

/** */
void DCHubListManager::AddBookmark( QString hubname, QString hubhost, QString description )
{
	if ( g_pConfig->AddBookmarkHub( hubname.toAscii().constData(), hubhost.toAscii().constData(), description.toAscii().constData() ) == TRUE )
	{
		g_pConfig->SaveDCBookHub();

		DCBookmarkHubItem * item = new DCBookmarkHubItem( TreeWidget_BOOKMARKS );
		item->setText( 0, hubname );
		item->setText( 1, hubhost );
		item->setText( 2, description );
		item->position = TreeWidget_BOOKMARKS->topLevelItemCount() - 1;

		// update tabbar
		TabWidget_SERVERLIST->setTabText( 1, tr("Bookmarks")+" ("+QString().setNum(TreeWidget_BOOKMARKS->model()->rowCount())+")" );
	}
}

/** */
void DCHubListManager::UpdateBookmark( QString hubname, QString hubhost, QString description )
{
	if ( g_pConfig->UpdateBookmarkHub( hubname.toAscii().constData(), hubhost.toAscii().constData(), description.toAscii().constData() ) == TRUE )
	{
		g_pConfig->SaveDCBookHub();
	}
}

/** */
CStringList * DCHubListManager::GetFilteredHubList()
{
	CStringList * list = 0;
	CObject * object;

	list = new CStringList();

	QList<PublicHubItem*> items = model->getAll();
	
	for ( int i = 0; i < items.size(); i++ )
	{
		CString addr = (items.at(i))->server.toAscii().constData();
		
		if ( list->Get( addr, (CObject *&)object ) != 0 )
		{
			list->Add( addr, new CString( addr ) );
		}
	}

	if ( list->Count() == 0 )
	{
		delete list;
		list = 0;
	}

	return list;
}

/** */
void DCHubListManager::slotContextMenuBookmarkList( const QPoint & )
{
	QAction * chosen = 0;
	QMenu *m;
	DCEditServer * des;
	QList<QTreeWidgetItem*> selitems = TreeWidget_BOOKMARKS->selectedItems();
	QTreeWidgetItem * curitem;
	DCConfigHubProfile pConfigHubProfile;
	DCConfigHubItem ConfigHubItem;

	CString profilename;

	m = new QMenu(this);

	QAction * connect = DCMenuHandler::addAction( m, emiCONNECT, (selitems.count() > 0) );
	DCMenuHandler::addAction( m, emiSEPARATOR );
	QAction * add = DCMenuHandler::addAction( m, emiADD );
	QAction * edit = DCMenuHandler::addAction( m, emiEDIT, (selitems.count() == 1) );
	QAction * remove = DCMenuHandler::addAction( m, emiREMOVE, (selitems.count() > 0) );
	DCMenuHandler::addAction( m, emiSEPARATOR );
	QAction * updserver = DCMenuHandler::addAction( m, emiUPDATE_SERVER, (selitems.count() > 0) );
	QAction * updall = DCMenuHandler::addAction( m, emiUPDATE_ALL_SERVER );
	DCMenuHandler::addAction( m, emiSEPARATOR );
	QAction * profiles = DCMenuHandler::addAction( m, emiHUB_PROFILE_EDITOR );

	chosen = m->exec(QCursor::pos());

	delete m;

	if ( chosen == 0 )
	{
		return;
	}

	if ( chosen == add )
	{
		des = new DCEditServer();
		des->setWindowTitle(tr("Add Bookmark"));

		//reset values
		des->Reset();
		
		while ( des->exec() == QDialog::Accepted )
		{
			// first we check some settings
			if ( des->LineEdit_NAME->text() == "" )
			{
				QMessageBox::critical( this, tr("Error"), tr("Please enter a hubname.") );

				continue;
			}

			if ( des->LineEdit_HOST->text() == "" )
			{
				QMessageBox::critical( this, tr("Error"), tr("Please enter a hubhost.") );

				continue;
			}

			// add bookmark to the list
			AddBookmark( des->LineEdit_NAME->text(), des->LineEdit_HOST->text(), des->LineEdit_DESCRIPTION->text() );

			// check if profiling possible
			if ( des->CheckBox_PROFILE->isChecked() == TRUE )
			{
				// get profile from dialog
				des->GetProfile(&pConfigHubProfile);
				
				// add new profile
				g_pConfig->AddHubProfile( &pConfigHubProfile );

				// save profile
				g_pConfig->SaveHubProfile();

				// set bookmark for that hub
				g_pConfig->SetBookmarkHubProfile(des->LineEdit_NAME->text().toAscii().constData(),des->LineEdit_NAME->text().toAscii().constData());
			}
			else
			{
				// clear bookmark from that hub
				g_pConfig->SetBookmarkHubProfile(des->LineEdit_NAME->text().toAscii().constData(),"");
			}
			
			// update myinfo
			g_pConnectionManager->SendMyInfoToConnectedServers();
			
			break;
		}

		delete des;
	}
	else if ( chosen == edit )
	{
		bool prof = FALSE;

		// edit only for one selected item
		curitem = selitems.first();

		des = new DCEditServer();
		des->setWindowTitle(tr("Edit Bookmark"));

		des->LineEdit_NAME->setText(curitem->text(0));
		des->LineEdit_HOST->setText(curitem->text(1));
		des->LineEdit_DESCRIPTION->setText(curitem->text(2));

		// hubitem
		if ( g_pConfig->GetBookmarkHub( curitem->text(0).toAscii().constData(), &ConfigHubItem ) == TRUE )
		{
			if ( ConfigHubItem.m_sProfile != "" )
			{
				// get profile
				if ( g_pConfig->GetHubProfile( ConfigHubItem.m_sProfile, &pConfigHubProfile ) == TRUE )
				{
					des->SetProfile(&pConfigHubProfile);

					prof = TRUE;
				}
			}
		}

		while ( des->exec() == QDialog::Accepted  )
		{
			// first we check some settings
			if ( des->LineEdit_NAME->text() == "" )
			{
				QMessageBox::critical( this, tr("Error"), tr("Please enter a hubname.") );

				continue;
			}

			if ( des->LineEdit_HOST->text() == "" )
			{
				QMessageBox::critical( this, tr("Error"), tr("Please enter a hubhost.") );

				continue;
			}

			RemoveBookmark( curitem );

			if ( prof == TRUE )
			{
				// remove the profile
				g_pConfig->DelHubProfile( pConfigHubProfile.m_sName );
                	}

			// add as new bookmark
			AddBookmark( des->LineEdit_NAME->text(), des->LineEdit_HOST->text(), des->LineEdit_DESCRIPTION->text() );

			// check if profiling possible
			if ( des->CheckBox_PROFILE->isChecked() == TRUE )
			{
				// get profile from dialog
				des->GetProfile(&pConfigHubProfile);

				// add new profile
				g_pConfig->AddHubProfile( &pConfigHubProfile );

				// save profile
				g_pConfig->SaveHubProfile();

				// set bookmark for that hub
				g_pConfig->SetBookmarkHubProfile(des->LineEdit_NAME->text().toAscii().constData(),des->LineEdit_NAME->text().toAscii().constData());
			}
			else
			{
				// remove bookmark from that hub
				g_pConfig->SetBookmarkHubProfile(des->LineEdit_NAME->text().toAscii().constData(),"");
			}

			// update myinfo
			g_pConnectionManager->SendMyInfoToConnectedServers();

			break;
		}

		delete des;
	}
	else if ( chosen == remove )
	{
		int i;
		StringMap * map;

		if ( g_pConfig->GetMap("DIALOG_REMOVE_BOOKMARK",map) == FALSE )
		{
			(*map)["SHOW"]    = QString().setNum(1);
			(*map)["DEFAULT"] = QString().setNum(0);
		}

		if ( ((*map)["SHOW"].toInt() == 1) || ((*map)["DEFAULT"].toInt() == 1) )
		{
			CDialogMessage * dlg = new CDialogMessage( this,
				QMessageBox::Warning,
				tr("Remove bookmarks"),
        			tr("You are sure ?"),
				tr("Remove"),
				tr("Cancel") );

			i = dlg->exec();

			(*map)["SHOW"]    = QString().setNum(!dlg->GetCheckBoxStatus());
			(*map)["DEFAULT"] = QString().setNum(i);

			delete dlg;


		}
		else
		{
			i = (*map)["DEFAULT"].toInt();
		}

		switch(i)
		{
	    		case 1: // The user clicked the cancel
        			return;
			default:
				break;
		}

	        for ( int i = 0; i < selitems.size(); i++ )
	        {
			curitem = selitems.at(i);
			
			// remove profile for that entry
			if ( g_pConfig->GetHubProfile( curitem->text(0).toAscii().constData(), &pConfigHubProfile ) == TRUE )
			{
				g_pConfig->DelHubProfile( pConfigHubProfile.m_sName );
			}

			// remove bookmark
			RemoveBookmark(curitem);
		}
		
		// save changes to profile list, some may have been deleted
		g_pConfig->SaveHubProfile();
		
		// update myinfo
		g_pConnectionManager->SendMyInfoToConnectedServers();
	}
	else if ( chosen == connect )
	{
	        for ( int i = 0; i < selitems.size(); i++ )
	        {
			curitem = selitems.at(i);
			
			g_pConnectionManager->Connect( curitem->text(0).toAscii().constData(), curitem->text(1).toAscii().constData() );
		}
	}
	else if ( chosen == updserver )
	{
		DCConfigHubItem hubitem;

	        for ( int i = 0; i < selitems.size(); i++ )
	        {
			curitem = selitems.at(i);
			
			if ( g_pConfig->GetPublicHub( curitem->text(0).toAscii().constData(), &hubitem ) == TRUE )
			{
				UpdateBookmark( hubitem.m_sName.Data(), hubitem.m_sHost.Data(), hubitem.m_sDescription.Data() );
				curitem->setText(1,hubitem.m_sHost.Data());
				curitem->setText(2,hubitem.m_sDescription.Data());
			}
		}
	}
	else if ( chosen == updall )
	{
		DCConfigHubItem * hubitem = 0;
		DCConfigHubItem hubitem1;
		CList<DCConfigHubItem> list;

		g_pConfig->GetBookmarkHubList(&list);

		while( (hubitem=list.Next(hubitem)) != 0 )
		{
			if ( g_pConfig->GetPublicHub( hubitem->m_sName, &hubitem1 ) == TRUE )
			{
				g_pConfig->UpdateBookmarkHub( hubitem1.m_sName, hubitem1.m_sHost, hubitem1.m_sDescription );
			}
		}

		g_pConfig->SaveDCBookHub();

		ShowBookmarkList();
	}
	else if ( chosen == profiles )
	{
		DCHubProfile * dlg = new DCHubProfile();

		dlg->exec();

		delete dlg;
		
		// update myinfo
		g_pConnectionManager->SendMyInfoToConnectedServers();
	}
}

/** */
void DCHubListManager::slotContextMenuPublicList( const QPoint & pos )
{
	QAction * chosen = 0;
	QMenu * m;
	QList<PublicHubItem*> selitems;
	PublicHubItem * curitem;
	int numSelected;
	
	QModelIndexList indexlist = TreeView_PUBLIC->selectionModel()->selectedIndexes();
	
	for ( int i = 0; i < indexlist.size(); i++ )
	{
		QModelIndex index = indexlist.at(i);
		
		curitem = model->getItem( index.row() );
		
		if ( selitems.contains( curitem ) == FALSE )
		{
			selitems.append( curitem );
		}
	}
	
	numSelected = selitems.size();
	
	if ( numSelected == 0 )
	{
		return;
	}

	m = new QMenu(this);

	QAction * connect = DCMenuHandler::addAction( m, emiCONNECT );
	DCMenuHandler::addAction( m, emiSEPARATOR );
	
	QAction * remove = 0;
	if ( g_pConfig->GetRemovePublicHubEnabled() )
	{
		remove = DCMenuHandler::addAction( m, emiREMOVE );
		DCMenuHandler::addAction( m, emiSEPARATOR );
	}
	
	QAction * bookmark = DCMenuHandler::addAction( m, emiADD_BOOKMARK );
	DCMenuHandler::addAction( m, emiSEPARATOR );
	QAction * copycol = DCMenuHandler::addAction( m, emiCOPY_COLUMN_TO_CLIPBOARD, (numSelected == 1) );
	QAction * copyrow = DCMenuHandler::addAction( m, emiCOPY_ROW_TO_CLIPBOARD, (numSelected == 1) );

	chosen = m->exec(QCursor::pos());

	delete m;

	if ( chosen == 0 )
	{
		return;
	}

	if ( chosen == connect )
	{
	        for ( int i = 0; i < selitems.size(); i++ )
	        {
			curitem = selitems.at(i);
			g_pConnectionManager->Connect( curitem->name.toAscii().constData(), curitem->server.toAscii().constData() );
		}
	}
	else if ( chosen == remove )
	{
		for ( int i = 0; i < selitems.size(); i++ )
		{
			curitem = selitems.at(i);
			
			CString hubname = curitem->name.toAscii().constData();
			CString hubhost = curitem->server.toAscii().constData();
			CString desc = curitem->description.toAscii().constData();
			
			if ( g_pConfig->RemovePublicHub(hubname,hubhost,desc) == TRUE )
			{
				model->deleteHub(curitem); // don't really need to check this return value too
			}
			else
			{
				QMessageBox::warning(this,tr("Remove public hub"),
				tr("Error removing hub \"") + QString::fromAscii(hubname.Data())
				+ QString("\" (") + QString::fromAscii(hubhost.Data()) + QString(")")
				);
			}
		}
		
		TabWidget_SERVERLIST->setTabText( 0, tr("Public")+" ("+QString().setNum(model->rowCount( QModelIndex() ))+"/"+QString().setNum(g_pConfig->GetPublicHubListSize())+")");
	}
	else if ( chosen == bookmark )
	{
	        for ( int i = 0; i < selitems.size(); i++ )
	        {
			curitem = selitems.at(i);
			AddBookmark( curitem->name, curitem->server, curitem->description );
		}

		TabWidget_SERVERLIST->setTabText( 1, tr("Bookmarks")+" ("+QString().setNum(TreeWidget_BOOKMARKS->model()->rowCount())+")" );
	}
	else if ( chosen == copycol )
	{
		int column = TreeView_PUBLIC->columnAt( pos.x() );
		
		curitem = selitems.first();
		
		QString s;
		
		switch ( column )
		{
			case 0:	s = curitem->name;
				break;
			case 1: s = curitem->server;
				break;
			case 2: s = curitem->description;
				break;
			case 3: s = QString().setNum(curitem->users);
				break;
			case 4: s = curitem->country;
				break;
			case 5: s = DCGuiUtils::GetSizeString(curitem->shared);
				break;
			case 6: s = DCGuiUtils::GetSizeString(curitem->minshare);
				break;
			case 7: s = curitem->extra;
				break;
			default: s = "";
				 break;
		}
		
		QApplication::clipboard()->setText( s );
	}
	else if ( chosen == copyrow )
	{
		QString s;

		curitem = selitems.first();
		
		s = curitem->name + " " + curitem->server + " "
		  + curitem->description + " " + QString().setNum(curitem->users)
		  + " " + curitem->country + " " + DCGuiUtils::GetSizeString(curitem->shared)
		  + " " + DCGuiUtils::GetSizeString(curitem->minshare)
		  + " " + curitem->extra;

		QApplication::clipboard()->setText(s);
	}
}

/** update CConfig's sorted bookmarks map */
void DCHubListManager::updateBookmarkOrder()
{
	if ( !(TreeWidget_BOOKMARKS->isSortingEnabled()) )
	{
		//printf("updateBookmarkOrder cancelled, widget is not sorted\n");
		return;
	}
	
	std::vector<int> newpositions(TreeWidget_BOOKMARKS->topLevelItemCount());
	
	int i = 0;
	QTreeWidgetItemIterator it( TreeWidget_BOOKMARKS );
	for ( ; (*it); ++it )
	{
		DCBookmarkHubItem * item = (DCBookmarkHubItem*) (*it);
		
		//printf("%s %d->%d\n",item->text(0).toAscii().constData(),item->position,i);
		if ( (item->position < 0) || (item->position >= newpositions.size()) )
		{
			printf("updateBookmarkOrder: Existing key for %s out of range at %d\n", item->text(0).toAscii().constData(), item->position);
			return;
		}
		newpositions[item->position] = i;
		item->position = i;
		
		++i;
	}
	
	//printf("call ReorderBookmarkHubs\n");
	
	g_pConfig->ReorderBookmarkHubs( newpositions );
	g_pConfig->SaveDCBookHub();
}

/** */
void DCHubListManager::slotMoveBookmarkUp()
{
	QTreeWidgetItem * item = TreeWidget_BOOKMARKS->currentItem();
	
	if ( !item )
	{
		return;
	}
	
	int oldindex = TreeWidget_BOOKMARKS->indexOfTopLevelItem( item );
	
	if ( oldindex < 1 )
	{
		return;
	}
	
	if ( TreeWidget_BOOKMARKS->isSortingEnabled() )
	{
		updateBookmarkOrder();
		TreeWidget_BOOKMARKS->setSortingEnabled(FALSE);
	}
	
	int newindex = oldindex - 1;
	
	TreeWidget_BOOKMARKS->takeTopLevelItem( oldindex );
	TreeWidget_BOOKMARKS->insertTopLevelItem( newindex, item );
	TreeWidget_BOOKMARKS->setCurrentItem( item );
	
	((DCBookmarkHubItem*) item)->position = newindex;
	((DCBookmarkHubItem*) TreeWidget_BOOKMARKS->topLevelItem( oldindex ))->position = oldindex;
	
	g_pConfig->MoveBookmarkHub( oldindex, newindex );
	
	g_pConfig->SaveDCBookHub();
}

/** */
void DCHubListManager::slotMoveBookmarkDown()
{
	QTreeWidgetItem * item = TreeWidget_BOOKMARKS->currentItem();
	
	if ( !item )
	{
		return;
	}
	
	int oldindex = TreeWidget_BOOKMARKS->indexOfTopLevelItem( item );
	
	if ( (oldindex == -1) || (oldindex > (TreeWidget_BOOKMARKS->topLevelItemCount() - 2)) )
	{
		return;
	}
	
	if ( TreeWidget_BOOKMARKS->isSortingEnabled() )
	{
		updateBookmarkOrder();
		TreeWidget_BOOKMARKS->setSortingEnabled(FALSE);
	}
	
	int newindex = oldindex + 1;
	
	TreeWidget_BOOKMARKS->takeTopLevelItem( oldindex );
	TreeWidget_BOOKMARKS->insertTopLevelItem( newindex, item );
	TreeWidget_BOOKMARKS->setCurrentItem( item );
	
	((DCBookmarkHubItem*) item)->position = newindex;
	((DCBookmarkHubItem*) TreeWidget_BOOKMARKS->topLevelItem( oldindex ))->position = oldindex;
	
	g_pConfig->MoveBookmarkHub( oldindex, newindex );
	
	g_pConfig->SaveDCBookHub();
}

/** */
void DCHubListManager::slotEnableBookmarkSorting()
{
	if ( TreeWidget_BOOKMARKS->isSortingEnabled() )
	{
		return;
	}
	else
	{
		TreeWidget_BOOKMARKS->setSortingEnabled(TRUE);
		// no updateBookmarkOrder() here
	}
}

/** */
void DCHubListManager::slotSaveBookmarks()
{
	/*
	 * Of couse updateBookmarkOrder() could just have been made a slot, but due to the large
	 * number of problems encountered while attempting to have the code in updateBookmarkOrder
	 * called automatically after the QTreeWidget has been sorted ( which appears to be impossible -
	 * you can connect to QAbstractItemModel::layoutChanged() or various QHeaderView signals
	 * but these are called either before the QTreeWidget has been sorted, or worse, during )
	 * so it is not a slot to indicate that it is not called automatically and updateBookmarkOrder()
	 * must be called manually at various points and on shutdown.
	 */
	if ( TreeWidget_BOOKMARKS->isSortingEnabled() )
	{
		updateBookmarkOrder();
	}
	else
	{
		g_pConfig->SaveDCBookHub();
	}
}
